@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G.pm,v 1.4 2002/07/30 17:40:56 gaou Exp $
@@ -33,6 +33,34 @@ use strict;
 use base qw(Exporter);
 use File::Find;
 
+#:::::::::::::::::::::::::::::::::
+#       Perldoc
+#:::::::::::::::::::::::::::::::::
+
+
+=head1 NAME
+
+    G::DynamicLoader - Dynamic Loading of Plug-in modules
+
+=head1 DESCRIPTION
+
+    This class is a part of G-language Genome Analysis Environment, 
+    for internal functionality of dynamically loading plug-in modules.
+
+    G-language System searches for plug-in modules located at 
+    ~/Plug-in or $ENV{GLANG_PLUGIN} at compile time, and exports 
+    all EXPORTed methods in any depths of folders within the plug-in 
+    directory. Therefore, users can customize and override the
+    installed G-language modules by placing the custom module 
+    in their plug-in folder with the same class name and method name.
+
+=cut
+
+
+#:::::::::::::::::::::::::::::::::
+#       Let the code begin...
+#:::::::::::::::::::::::::::::::::
+
 our @EXPORT;
 
 BEGIN{
@@ -70,14 +98,14 @@ BEGIN{
 		    eval "use $dirname";
 		    warn "$@" if ($@);
 		    
-		    push(@EXPORT, @{"$dirname" . '::' . "EXPORT"});	   
+		    push(@EXPORT, @{$dirname . '::' . 'EXPORT'});	   
 		}
 	    }
 	    closedir(DIR);
 	}
     };
     
-    find($sub, "$path") if (-e $path);
+    find($sub, $path) if (-e $path);
 }
 
 1;
