@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: G::IO.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -41,11 +41,13 @@ require G::IO::FastaI;
 require G::IO::FastQI;
 require G::IO::EmblI;
 
+use Carp;
+use LWP::Simple;
 use File::Basename;
 use File::ShareDir ':ALL';
 use autouse 'File::Temp'=>qw(tempfile);
 
-our $VERSION = '1.8.10';
+our $VERSION = '1.8.11';
 
 #::::::::::::::::::::::::::::::
 #          Perldoc
@@ -78,8 +80,7 @@ Kazuharu Arakawa, gaou@sfc.keio.ac.jp
 
 my $outfile = '';
 my $loaded = 0;
-my $infile = '';
-my $this = {};
+my @instances;
 
 #::::::::::::::::::::::::::::::
 #    Let the code begin...
@@ -105,42 +106,37 @@ sub new {
     }
 
     my @options = @_;
-    $this = {};
+    my $this = {};
     my $tmp = {};
-    $infile = $filename;
+
     $filename =~ s/\~\//sprintf("%s\/",$ENV{HOME})/e;
     
     bless $this;
 
     my $no_msg                 = 0;
-    my $without_annotation     = 0;
     my $multiple_locus         = 0;
-    my $long_sequence          = 0;
     my $bioperl                = 0;
     my $netgbk                 = 0;
     my $longest_ORF_annotation = 0;
     my $glimmer_annotation     = 0;
     my $no_cache               = 0;
     my $force_cache            = 0;
+    my $gzip                   = 0;
     my $format                 = '';
     my $locus_msg              = '';
 
     return $this if ($filename eq 'blessed');
 
+    $gzip = $filename =~ s/\.gz$//;
+    
     foreach my $opt_tmp (@options){
 	my $opt = lc($opt_tmp);
 
 	if ($opt =~ /no msg/){
 	    $no_msg = 1;
-	}elsif ($opt =~ /without annotation/){
-	    $without_annotation = 1;
-	    $no_cache = 1;
 	}elsif ($opt =~ /multiple locus/){
 	    $multiple_locus = 1;
 	    $no_cache = 1;
-	}elsif ($opt =~ /long sequence/){
-	    $long_sequence = 1;
-	    $no_cache = 1;
 	}elsif ($opt =~ /bioperl/){
 	    $bioperl = 1;
 	    $format = 'bioperl';
@@ -164,9 +160,6 @@ sub new {
 		|| $opt eq 'qual'  || $opt eq 'bsml' 
 		|| $opt eq 'fastq' || $opt eq 'genbank' ){
 	    $format = $opt;
-	    if($opt eq 'fastq' || $opt eq 'fasta'){
-		$no_cache = 1;
-	    }
 	}else{
 	    unless (lc($opt) eq 'net genbank'){
 		warn("Warning: Unknown Option $opt at \"new G\"\n");
@@ -180,13 +173,17 @@ sub new {
 
     if($format eq 'RefSeq'){
 	$format = 'usa';
-	$filename = "refseq:$filename";
+	$filename = "genbank:$filename";
     }elsif($format eq 'net GenBank'){
 	$format = 'usa';
 	$filename = "genbank:$filename";
     }elsif($format eq 'net GenPept'){
 	$format = 'usa';
-	$filename = "genpept:$filename";
+	$filename = "genbank:$filename";
+    }
+    
+    if($format eq 'fastq' || $format eq 'fasta'){
+	$no_cache = 1 unless($force_cache);
     }
 
     my @stat = stat($filename);
@@ -204,22 +201,39 @@ sub new {
 
     }elsif ($format eq 'usa'){
 	my ($dbname, $entryname) = split(/:/, $filename, 2);
+	msg_error("Retrieving sequence $entryname from " . uc($dbname) . "...\n");
 
-	msg_error("Retrieving sequence from " . uc($dbname) . "...\n");
+	if($dbname eq 'sw' || $dbname eq 'uniprot'){
+	    $dbname = 'swiss';
+	}
 
-	my $bp;
+	if($dbname eq 'refseq'){
+	    $dbname = 'genbank';
+	}
 
-	eval {
-	    $bp = get_sequence($dbname, $entryname);
-	};
-	die("$@ Could not retrieve $filename\n") if $@;
-
-        $this = _bp2gb($bp);
-	$this->seq_info() unless($no_msg);
-	$this->{BIOPERL} =  $bp;
+	if($dbname eq 'genbank' || $dbname eq 'embl'){
+	    my $path = _sdb_path() . $entryname . '.' . $dbname;
+	    mirror('http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname, $path);
+	    if($no_msg){
+		$this = new G::IO($path, 'no msg');
+	    }else{
+		$this = new G::IO($path);
+	    }
+	    croak('Sequence not found in database at http://togows.dbcls.jp/entry/' . $dbname . '/'. $entryname) unless(length($this->{SEQ}));
+	}else{
+	    my $bp;
+	    
+	    eval {
+		$bp = get_sequence($dbname, $entryname);
+	    };
+	    die("$@ Could not retrieve $filename\n") if $@;
+	    
+	    $this = _bp2gb($bp);
+	    $this->seq_info() unless($no_msg);
+	    $this->{BIOPERL} =  $bp;
+	}
     }else{
 
-
 	if ($format eq 'embl'){
 	    $this = new G::IO::EmblI;
 	}elsif ($format eq 'fasta'){
@@ -237,7 +251,7 @@ sub new {
 	    $format = 'genbank';
 	}
 
-	*GENBANK = $this->open_gb($filename);
+	my $fh = $this->open_gb($filename . ($gzip ? '.gz' : ''));
 
 	if ($multiple_locus){
 	    my $tmp = $this->clone();
@@ -248,25 +262,11 @@ sub new {
 	    $this->set_gene_aliases();
 	    $this->seq_info() unless($no_msg);
 	}else{
-	    if ($without_annotation){
-		$this->goto_origin();
-		$this->getnucs();
-		$this->seq_info() unless ($no_msg);
-		warn("This feature is deprecated. It will be removed in future releases.");
-	    }elsif ($long_sequence){
-		$this->read_locus();
-		$this->read_header();
-		$this->read_features();
-		$this->{origin} = tell *GENBANK;
-		warn("This feature is deprecated. It will be removed in future releases.");
-	    }else{
-		$this->next_locus($no_msg);
+	    $this->next_locus($no_msg);
 
-		if($force_cache || $no_cache == 0){
-		    sdb_save($this, sprintf("cache-%s-%s-%s", $basename, $stat[7], $stat[9]));
-		}
+	    if($force_cache || $no_cache == 0){
+		sdb_save($this, sprintf("cache-%s-%s-%s", $basename, $stat[7], $stat[9]));
 	    }
-
 	}
     }
 
@@ -287,17 +287,14 @@ sub new {
 	return $new;
     }
 
+    shift @instances;
+    push(@instances, $this);
+
     return $this;
 }
 
 sub lastInstance {
-    return $this;
-}
-
-sub filepath {
-    my $this = shift;
-
-    return $infile;
+    return $instances[0];
 }
 
 sub loaded_msg {
@@ -321,7 +318,7 @@ sub loaded_msg {
 	         Arakawa K. et al. (2008) Genes, Genomes and Genomics.
 
  	      License: GNU General Public License
-	      Copyright (C) 2001-2009 G-language Project
+	      Copyright (C) 2001-2010 G-language Project
 	      Institute for Advanced Biosciences, Keio University, JAPAN 
 
 	     __/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/__/
@@ -402,15 +399,5 @@ sub multi_locus{
     $this->{"COMMENT"} = $this->{"COMMENT1"};
 }
 
-sub DESTROY {
-    my $self = shift;
-
-    undef %{$self};
-    $self->close_gb();
-    unlink($outfile);
-}
-
-
-
 1;
 
