@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GenBankI.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -32,75 +32,60 @@ package G::IO::EmblI;
 use strict;
 use base qw(G::IO::Handler G::IO::GenBankO);
 
+use Carp;
+use IO::File;
+use IO::Zlib;
 use G::Messenger;
 
-my $infile;
-
 #::::::::::::::::::::::::::::::
 #     Let the code begin...
 #::::::::::::::::::::::::::::::
 
+my %FH;
+
 sub new{
     my $pkg = shift;
-    my $filename = shift;
-    my $option = shift;
     my $this = {};
 
-    bless $this;
-
-    return $this;
+    return bless $this;
 }
 
-sub filepath {
-    my $this = shift;
-
-    return $infile;
+sub DESTROY {
+    return unless($FH{$_[0]->{INSTANCE}});
+    close ($FH{$_[0]->{INSTANCE}});
+    delete($FH{$_[0]->{INSTANCE}});
 }
 
 sub open_gb {
     my $this = shift;
     my $filename = shift;
-    $infile = $filename;
-    
-    open(EMBL, $filename) || die("Error at G::IO::EmblI: $!\n");
-    return *EMBL;
-}
+    $this->{FILENAME} = $filename;
+    $this->{INSTANCE} = $filename . '-' . time() . '-' . rand();
+    $FH{$this->{INSTANCE}} = new IO::File;
 
-sub close_gb {
-    my $this = shift;
-    close(EMBL);
-}
-
-sub next_seq{
-    my $this = shift;
-    my $len = '100';
-    my $opt = shift;
-    my $char = '1';
-    $len = $opt if ($opt);
-    $this->{SEQ} = '';
-
-    while($len > 0 && $char ne ''){
-	$char = getc(EMBL);
-	next unless ($char =~ /[a-zA-Z]/);
-	$len --;
-	$this->{SEQ} .= lc($char);
+    if($filename =~ /\.gz$/){
+        $FH{$this->{INSTANCE}} = new IO::Zlib;
+        $FH{$this->{INSTANCE}}->open($filename, 'rb') or croak("Error at G::IO::EmblI: $!\n");;
+    }else{
+        $FH{$this->{INSTANCE}} = new IO::File;
+        open($FH{$this->{INSTANCE}}, $filename) or croak("Error at G::IO::EmblI: $!\n");
     }
-    $this->{position} = tell EMBL;
 
-    return $char;
-}
-
-sub rewind_genome{
-    my $this = shift;
-    seek EMBL, $this->{origin}, 0;
-    return 1;
+    return $FH{$this->{INSTANCE}};
 }
 
 sub next_locus{
     my $this = shift;
     my $msg = shift;
+
+    my $file = $this->{FILENAME};
+    my $inst = $this->{INSTANCE};
+
     undef %{$this};
 
+    $this->{FILENAME} = $file;
+    $this->{INSTANCE} = $inst;
+
     my $ret = $this->read_locus($msg);
     $ret = $this->read_features() if($ret < 50);
     $ret = $this->getnucs() if($ret < 99);
@@ -114,26 +99,13 @@ sub next_locus{
     }
 }
 
-sub goto_features{
-    my $this = shift;
-    while(<EMBL>){
-	last if (/^FH/);
-    }
-}
-
-sub goto_origin{
-    my $this = shift;
-    while(<EMBL>){
-	last if (/^SQ/);
-    }
-}
-
 sub read_locus{
     my $this = shift;
     my $msg = shift;
     local($_);
 
-    while(<EMBL>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
 	$this->{COMMENT} .= $_;
 
 	if(/^FH/ || /^FT/){
@@ -181,10 +153,6 @@ sub read_locus{
     }
 }
 	
-sub read_header{
-    my $this = shift;
-}
-
 sub read_features{
     local($_);
     my $this = shift;
@@ -192,9 +160,15 @@ sub read_features{
     my $cds = 0;
     $this->{"CDS0"}->{dummy} = 1;
 
-    while(<EMBL>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
+        my $tmpline;
+        JUMPLABEL: if(length($tmpline)){
+	    $_ = $tmpline;
+        };
+
 	next if (/^XX/ || /^FH/);
-	my $tmpline = $_;
+	my $tmpline2 = $_;
 
 	if(/^\/\//){
 	    return 99;
@@ -230,17 +204,13 @@ sub read_features{
 		    $this->{"FEATURE$num"}->{"direction"} = "direct";
 		}
 
-		my $line = $_;
-		my $fth = '';
-		my $linenum = tell EMBL;
+                my $fth = $_;
 
-		while($line !~ /\//){
-		    $fth .= substr($line, 2);
-		    $line = <EMBL>;
-		    $linenum = tell EMBL;
-		    $linenum -= length($line);
-		}
-		seek EMBL, $linenum, 0;
+                while(<$fh>){
+                    $tmpline = $_;
+                    last if($tmpline =~ /\// || $tmpline =~ /ORIGIN/ || $tmpline =~ /^ {5}\S+/);
+                    $fth .= $tmpline;
+                }
 
 		substr($fth, 0, 19) = '';
 		$fth =~ s/join//g;
@@ -257,8 +227,8 @@ sub read_features{
 		$this->{"FEATURE$num"}->{"end"} = pop @choparray;
 		$this->{"FEATURE$num"}->{"join"} = $fth;
 
-		if ($line =~ /\:/){
-		    $this->{"FEATURE$num"}->{"partial"} = $line;
+		if ($fth =~ /\:/){
+		    $this->{"FEATURE$num"}->{"partial"} = $fth;
 		}
 	    }elsif (/\?/){
 		$this->{"FEATURE$num"}->{"type"} = "partial_$key";
@@ -307,7 +277,7 @@ sub read_features{
 		my $tag = $1;
 		my $tmp = $2;
 		my $line;
-		while(<EMBL>){
+		while(<$fh>){
 		    substr($_, 0, 2) = '';
 		    if (!/\"/){
 			$tmp .= $_;
@@ -328,8 +298,8 @@ sub read_features{
 	    }elsif (/\/(\w+)/){
 		$this->{"FEATURE$num"}->{$1} .= '1';
 	    }else{
-		chomp($tmpline);
-		msg_error("Irregular feature line: $tmpline") unless(/^FT/);
+		chomp($tmpline2);
+		msg_error("Irregular feature line: $tmpline2") unless(/^FT/);
 	    }
 	}
     }
@@ -340,7 +310,8 @@ sub read_features{
 sub getnucs {
     my $this = shift;
 
-    while(<EMBL>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
 	return if (/^\/\//);
 	next if (/^\S/);
 	s/[^A-Za-z]//g;
