@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: GenBankI.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -30,63 +30,52 @@
 package G::IO::GenBankI;
 
 use strict;
+use Carp;
+use IO::File;
+use IO::Zlib;
 use G::Messenger;
 
-my $infile;
-
 #::::::::::::::::::::::::::::::
 #     Let the code begin...
 #::::::::::::::::::::::::::::::
 
-sub filepath {
-    my $this = shift;
-
-    return $infile;
-}
+my %FH;
 
 sub open_gb {
     my $this = shift;
     my $filename = shift;
-    $infile = $filename;
-    
-    open(GENBANK, $filename) || die("Error at G::IO::GenBankI: $!\n");
-    return *GENBANK;
-}
+    $this->{FILENAME} = $filename;
+    $this->{INSTANCE} = $filename . '-' . time() . '-' . rand();
 
-sub close_gb {
-    my $this = shift;
-    close(GENBANK);
-}
-
-sub next_seq{
-    my $this = shift;
-    my $len = '100';
-    my $opt = shift;
-    my $char = '1';
-    $len = $opt if ($opt);
-    $this->{SEQ} = '';
-
-    while($len > 0 && $char ne ''){
-	$char = getc(GENBANK);
-	next unless ($char =~ /[a-zA-Z]/);
-	$len --;
-	$this->{SEQ} .= lc($char);
+    if($filename =~ /\.gz$/){
+        $FH{$this->{INSTANCE}} = new IO::Zlib;
+        $FH{$this->{INSTANCE}}->open($filename, 'rb') or croak("Error at G::IO::GenBankI: $!\n");;
+    }else{
+        $FH{$this->{INSTANCE}} = new IO::File;
+        open($FH{$this->{INSTANCE}}, $filename) or croak("Error at G::IO::GenBankI: $!\n");
     }
-    $this->{position} = tell GENBANK;
 
-    return $char;
+    return $FH{$this->{INSTANCE}};
 }
 
-sub rewind_genome{
-    my $this = shift;
-    seek GENBANK, $this->{origin}, 0;
-    return 1;
+sub DESTROY {
+    return unless($FH{$_[0]->{INSTANCE}});
+    close ($FH{$_[0]->{INSTANCE}});
+    delete($FH{$_[0]->{INSTANCE}});
 }
 
 sub next_locus{
     my $this = shift;
-    my $msg = shift;
-    undef %{$this};
+    my $msg  = shift;
+
+    my $file = $this->{FILENAME};
+    my $inst = $this->{INSTANCE};
+
+    undef %$this;
+
+    $this->{FILENAME} = $file;
+    $this->{INSTANCE} = $inst;
+
     $this->read_locus($msg);
     $this->read_header();
     $this->read_features();
@@ -101,27 +90,14 @@ sub next_locus{
     }
 }
 
-sub goto_features{
-    my $this = shift;
-    while(<GENBANK>){
-	last if (/^FEATURES/);
-    }
-}
-
-sub goto_origin{
-    my $this = shift;
-    while(<GENBANK>){
-	last if (/^ORIGIN/);
-    }
-}
-
 sub read_locus{
     my $this = shift;
     my $msg = shift;
     my $tmp = '';
     local($_);
 
-    while(<GENBANK>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
 	next unless (/LOCUS/);
 
 	my @locus_line = split;
@@ -157,7 +133,8 @@ sub read_header{
     my $this = shift;
     my $line = '';
 
-    while($line = <GENBANK>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while($line = <$fh>){
 	last if ($line =~ /^FEATURES/);
 	$this->{HEADER} .= $line;
     }
@@ -205,7 +182,13 @@ sub read_features{
     my $transexc = 0;
     $this->{"CDS0"}->{dummy} = 1;
 
-    while(<GENBANK>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
+	my $tmpline;
+        JUMPLABEL: if(length($tmpline)){
+	    $_ = $tmpline;
+	};
+
 	if (/^BASE COUNT/){
 	    s/BASE COUNT //g;
 	    $this->{"BASE_COUNT"} = $_;
@@ -241,18 +224,13 @@ sub read_features{
 		    $this->{"FEATURE$num"}->{"direction"} = "direct";
 		}
 
-		my $line = $_;
-		my $fth = $line;
-		my $linenum = tell GENBANK;
+		my $fth = $_;
 
-		while(<GENBANK>){
-		    $line = $_;
-		    $linenum = tell GENBANK;
-		    $linenum -= length($line);
-		    last if($line =~ /\// || $line =~ /ORIGIN/ || $line =~ /^ {5}\S+/);
-		    $fth .= $line;
+		while(<$fh>){
+		    $tmpline = $_;
+		    last if($tmpline =~ /\// || $tmpline =~ /ORIGIN/ || $tmpline =~ /^ {5}\S+/);
+		    $fth .= $tmpline;
 		}
-		seek GENBANK, $linenum, 0;
 
 		substr($fth, 0, 19) = '';
 		$fth =~ s/join//g;
@@ -262,16 +240,28 @@ sub read_features{
 		$fth =~ s/\n//g;
 		$fth =~ s/complement/c/g;
 
+		my @tmpfth2;
+		for my $field (split(/,/, $fth)){
+		    if($field =~ /\.\./){
+			push(@tmpfth2, $field);
+		    }else{
+			push(@tmpfth2, "$field..$field");
+		    }
+		}
+		$fth = join(',', @tmpfth2);
 		my $tmpfth = $fth;
 		$tmpfth =~ s/c//g;
+
 		my @choparray = split(/\.\./, $tmpfth);
 		$this->{"FEATURE$num"}->{"start"} = shift @choparray;
 		$this->{"FEATURE$num"}->{"end"} = pop @choparray;
 		$this->{"FEATURE$num"}->{"join"} = $fth;
 
-		if ($line =~ /\:/){
-		    $this->{"FEATURE$num"}->{"partial"} = $line;
+		if ($fth =~ /\:/){
+		    $this->{"FEATURE$num"}->{"partial"} = $fth;
 		}
+
+		goto JUMPLABEL;
 	    }elsif (/\?/){
 		$this->{"FEATURE$num"}->{"type"} = "partial_$key";
 		$this->{"FEATURE$num"}->{"partial"} = $feature;
@@ -328,7 +318,8 @@ sub read_features{
 		my $tag = $1;
 		my $tmp = $2;
 		my $line;
-		while(<GENBANK>){
+
+		while(<$fh>){
 		    if (!/\"/){
 			$tmp .= $_;
 		    }elsif (/([^\"]+)\"/){
@@ -369,7 +360,8 @@ sub read_features{
 sub getnucs {
     my $this = shift;
 
-    while(<GENBANK>){
+    my $fh = $FH{$this->{INSTANCE}};
+    while(<$fh>){
 	last if (/\/\//);
 	s/[^A-Za-z]//g;
 	$this->{"SEQ"} .= lc($_);
