@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -94,7 +94,8 @@ sub say{
    This method simply reads all contents of a file and returns
    the contents as a scalar or an array of lines depending on the
    context. Specify second argument as 1 to chomp each lines.
-   The file can be local, or remote (starting with http:// or ftp://).
+   The file can be local, remote (starting with http:// or ftp://),
+   or gzipped.
 
   Usage: 
     #in array context
@@ -118,11 +119,14 @@ sub say{
                  'fasta' for FASTA file, 'fastq' for FASTQ file.
                  Usually the format is automatically interpreted, but
                  this option can be specified to force interpretation.
+    -gzip        0 for uncompressed files, 1 for gzipped files (default:0)
+                 automatically turned on when filename ends with ".gz"
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20100122-01 added support for gzipped files
     20091202-01 added FastA/FastQ support
     20090520-01 added URL support
     20090313-01 initial posting
@@ -130,11 +134,12 @@ sub say{
 =cut
 
 sub readFile{
-    opt_default('format'=>'');
+    opt_default('format'=>'', 'gzip'=>0);
     my @argv    = opt_get(@_);
     my $file    = shift @argv;
     my $chomp   = shift @argv || 0;
     my $format  = lc(opt_val('format'));
+    my $gzip    = lc(opt_val('gzip'));
 
     unless($format){
 	if (lc($file) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
@@ -163,12 +168,21 @@ sub readFile{
 
 	return %hash;
     }else{
+	my $fh;
+	if($file =~ /\.gz$/ || $gzip){
+	    $fh = new IO::Zlib;
+	    $fh->open($file, 'rb') or croak($!);
+	}else{
+	    $fh = new IO::File;
+	    open($fh, $file) or croak($!);
+	}
+
 	open(FILE, $file) || croak($!);
-	while(<FILE>){
+	while(<$fh>){
 	    chomp if($chomp);
 	    push(@result, $_);
 	}
-	close(FILE);
+	close($fh);
     }
 
     if(wantarray()){
