@@ -482,6 +482,80 @@ sub _codon_usage_table {
 }
 
 
+sub codon_amino_counter{
+    my $gb=shift;
+    my $key=shift;
+    my %result;
+    my $seq;
+    my $amino;
+    my $codon;
+    my $num;
+    my $q;
+    my @tmp;
+    
+    if(defined(%{$gb->{"$key"}})){
+	$seq=lc($gb->get_geneseq("$key"));
+	$num=$gb->{"$key"}->{feature};
+    }
+    
+    @tmp=split(//,$gb->{"FEATURE$num"}->{translation});
+    for($q=0;$q<length($seq);$q+=3){
+	$amino=shift @tmp;
+	$amino="/" if($amino eq "");
+	$codon=substr($seq,$q,3);
+	$result{$amino}{$codon}++;
+    }
+    
+    return %result;
+}
+#codon_usage_printer ver.20010722-01
+#scripting by Koya Mori(mory@g-language.org)
+#This program prints result of codon usage.
+#&codon_usage_printer(pointer codon_usage,  boolean debug);
+sub _codon_usage_printer{
+    &opt_default(output=>"stdout");
+    my @args=opt_get(@_);
+    my $result=shift @args;
+    my $printer=opt_val("output");
+    my $filename=opt_val("filename");
+    my $amino_total;
+    my $key;
+    my $key2;
+    my $total;
+
+
+    if($printer eq "f"){
+	open(FILE,">>$filename");
+	foreach $key (sort keys(%$result)){
+	    $amino_total=0;
+	    foreach $key2 (sort keys(%{$$result{$key}})){
+		$amino_total+=$$result{$key}{$key2};
+	    }
+	    foreach $key2 (sort keys(%{$$result{$key}})){
+		print FILE $key,",",$key2,",",$$result{$key}{$key2},",",sprintf("%.3f",$$result{$key}{$key2}/$amino_total),"\n";
+		$total+=$$result{$key}{$key2};
+	    }
+	}
+	print FILE "total,$total\n";
+	print FILE "\n\n";
+	close(FILE);
+    }
+    else{
+	foreach $key (sort keys(%$result)){
+	    $amino_total=0;
+	    foreach $key2 (sort keys(%{$$result{$key}})){
+		$amino_total+=$$result{$key}{$key2};
+	    }
+	    foreach $key2 (sort keys(%{$$result{$key}})){
+		&msg_send($key," -> ",$key2," -> ",$$result{$key}{$key2},"   ",sprintf("%.3f",$$result{$key}{$key2}/$amino_total),"\n");
+		$total+=$$result{$key}{$key2};
+	    }
+	}
+	&msg_send("total -> $total\n");
+    }
+}
+
+
 =head2 enc
 
   Name: enc   -   calculate the effective number of codons (Nc)
@@ -2367,7 +2441,7 @@ sub codon_usage{
 	$usage{'/'}{tag} = 0.0001;
 
 	$option{'-display'} = 'number';
-    }elsif($key){
+    }elsif(length $key){
 	%usage=codon_amino_counter($gb,$key);
     }else{
 	foreach($gb->cds()){
