@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: PatSearch.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -35,9 +35,11 @@ use G::Seq::Primitive;
 
 use strict;
 use base qw(Exporter);
+use Carp;
 use SelfLoader;
 
 our @EXPORT = qw(
+	     markov
 	     signature
 	     signature_dist	 
 	     palindrome
@@ -80,6 +82,170 @@ __DATA__
 
 
 
+=head2 markov
+
+  Name: markov   -   calculate O/E values of oligonucleotides with Markov statistics
+
+  Description:
+    This program calculates the Markov probability for all oligomers of specified length.
+    Returned value is a reference to a hash with keys as oligomer 
+    sequences, and valus as the O/E values at (length - 2) order Markov.
+    
+  Usage:
+    $ref_hash = &markov(pointer G instance);  
+
+  Options:
+   -length    length of oligomer to analyze (default:6)
+   -mincount  minimum number of oligomer count to report (default:10)
+   -filename  output filename (default:'markov.csv')
+   -output    "f" for file output, "stdout" for STDOUT output
+
+  Author: 
+   Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+   20010830-01 initial posting
+
+=cut
+
+sub markov {
+    &opt_default(length=>6, mincount=>10, filename=>"markov.csv",output=>"stdout");
+    my @args = opt_get(@_);
+    my $gb = opt_as_gb(shift @args);
+    my $filename = opt_val("filename");
+    my @aSortedTable = ( );
+    my $iTotalNucs = 0;
+    my @ahNucsTable = ( );
+    my %oe;
+
+    for (my $iCounter = 0; $iCounter <= 32; $iCounter++) {
+	$ahNucsTable[$iCounter] = { };
+    }
+
+    my $rhTmp;
+    foreach $rhTmp (@ahNucsTable) { undef %$rhTmp; }
+    $iTotalNucs = 0;
+
+    my($nucs, $char);
+    $nucs = '';
+    foreach $char (split(//, $gb->{SEQ})) {
+	$iTotalNucs++;
+	$nucs .= $char;
+	if (opt_val("length") < $iTotalNucs) {
+	    substr($nucs, 0, 1) = '';
+	}
+	;# Now $nucs contains tail of sequence.
+	my $iLoopEnd = opt_val("length");
+	if ($iTotalNucs < $iLoopEnd) {
+	    $iLoopEnd = $iTotalNucs;
+	}
+	my $iLen;
+	for ($iLen = 1; $iLen <= $iLoopEnd; $iLen++) {
+	    $ahNucsTable[$iLen - 1]->{substr($nucs, -$iLen, $iLen)}++;
+	}
+    }
+
+    {
+	my @aTmpTable1 = ( );
+	my @aTmpTable2 = ( );
+	my @aTmpTable3 = ( );
+	my $sKey;
+	foreach $sKey (keys(%{$ahNucsTable[opt_val("length") - 1]})) {
+	    my $iTmp = $ahNucsTable[opt_val("length") - 1]->{$sKey};
+	    if (opt_val("mincount") <= $iTmp) {
+		my $sTmp = sprintf("%08d %s", $iTmp, $sKey);
+		if ($iTmp == 1) {
+		    push(@aTmpTable1, $sTmp);
+		} elsif ($iTmp == 2) {
+		    push(@aTmpTable2, $sTmp);
+		} else {
+		    push(@aTmpTable3, $sTmp);
+		}
+	    }
+	}
+	@aSortedTable = sort {$b cmp $a;} @aTmpTable3;
+	push(@aSortedTable, @aTmpTable2);
+	push(@aSortedTable, @aTmpTable1);
+    }
+
+    if (opt_val("output") eq "f"){
+	mkdir ('data', 0777);
+	open(TABLEFILE, '>data/' . $filename) || die;
+	print TABLEFILE "oligomer,O-value,E-value,";
+	my $i;
+	for ($i = 1; $i <= opt_val("length") - 2; $i ++){
+	    printf TABLEFILE "%d degree Markov,", $i;
+	}
+	print TABLEFILE "O/E value\n";
+    }
+
+    foreach my $sRecord (@aSortedTable) {
+	my($iOVal, $sKey) = split(' ', $sRecord);
+	my $klen = length($sKey);
+	$iOVal =~ s/^0+//;
+	my ($order, $iEVal);
+
+	if (opt_val("output") eq "f"){
+	    printf TABLEFILE "%s,%d,", $sKey, $iOVal;
+	}elsif(opt_val("output") eq "stdout"){
+	    &msg_send(sprintf("%s %5d", $sKey, $iOVal));
+	}
+
+	if (opt_val("length") == 1){
+	    if (opt_val("output") eq "f"){
+		printf TABLEFILE "\n";
+	    }elsif(opt_val("output") eq "stdout"){
+		&msg_send("\n");
+	    }
+	}else{
+	    for ($order = 0; $order <= opt_val("length") - 2; $order++) {
+		my $numerator = $iTotalNucs + 1 - $klen;
+		my $denominator = 1.0;
+		my $offset;
+		for ($offset = 0; $offset <= $klen - $order - 1; $offset++) {
+		    my $key = substr($sKey, $offset, $order + 1);
+		    my $len = length($key);
+		    $numerator *=  $ahNucsTable[$len - 1]->{$key} / 
+			($iTotalNucs + 1 - $len);
+		}
+		if (1 <= $order) {
+		    for ($offset = 1; $offset <= $klen - $order - 1; $offset++) {
+			my $key = substr($sKey, $offset, $order);
+			my $len = length($key);
+			$denominator *=  $ahNucsTable[$len - 1]->{$key} / 
+			    ($iTotalNucs + 1 - $len);
+		    }
+		} else {
+		    $denominator = 1.0;
+		}
+		if ($denominator <= 0.0) {
+		    $iEVal = 0.0;
+		} else {
+		    $iEVal = $numerator / $denominator;
+		}
+		
+		if (opt_val("output") eq "f"){
+		    printf TABLEFILE "%d,", $iEVal if (opt_val("output") eq "f");
+		}elsif(opt_val("output") eq "stdout"){
+		    &msg_send(sprintf(" %8d", $iEVal));
+		}
+	    }
+	    if (opt_val("output") eq "f"){
+		printf TABLEFILE "%.4f\n", $iOVal/$iEVal;
+	    }elsif(opt_val("output") eq "stdout"){
+		&msg_send(sprintf("   %3.4f\n", $iOVal/$iEVal));
+	    }
+	    $oe{$sKey} = $iOVal/$iEVal;
+	}
+    }
+    close(TABLEFILE) if (opt_val("output") eq "f");
+
+    return \%oe;
+}
+
+
+
+
 =head2 signature
 
   Name: signature   -   calculate oligonucleotide usage (genomic signature)
@@ -122,17 +288,14 @@ __DATA__
 
 sub signature {
     &opt_default(output=>'stdout', filename=>'signature.csv', wordlength=>2, start=>1, end=>0);
-    my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
-    my $output = opt_val("output");
-    my $filename = opt_val("filename");
+    my @args       = opt_get(@_);
+    my $gb         = opt_as_gb(shift @args);
+    my $output     = opt_val("output");
+    my $filename   = opt_val("filename");
     my $wordlength = opt_val("wordlength");
-    my $start = opt_val("start");
-    my $end = opt_val("end");
-    my %mono;
-    my %obs;
-    my %exp;
-    my %oe;
+    my $start      = opt_val("start");
+    my $end        = opt_val("end");
+    my (%mono, %obs, %exp, %oe);
 
     $end = length($gb->{SEQ}) unless($end);
     #my $seq1 = $gb->getseq($start, $end); # Perl coordinates
@@ -143,12 +306,17 @@ sub signature {
 	[acgturymkdhbvwsn]
 	[tgcaayrkmhdvbwsn];
 
-    $seqb = $seq1.$seq2;
+    my $seqb = $seq1.$seq2;
     $mono{a} = $seqb =~ tr/a/a/;
     $mono{c} = $seqb =~ tr/c/c/;
     $mono{g} = $seqb =~ tr/g/g/;
     $mono{t} = $seqb =~ tr/t/t/;
     my $total = $seqb =~ tr/acgt/acgt/;
+    unless($total){
+	carp("No regular nucleotide found in this sequence.");
+	return;
+    }
+
     foreach (keys %mono){ $mono{$_} =  $mono{$_} / $total; }
 
     for(my $i = 0; $i < length($seq1) - $wordlength + 1; $i ++){
@@ -206,17 +374,17 @@ sub signature {
     if($output eq 'f'){
 	#mkdir ("data", 0777);
 	#open(FILE,">>data/$filename");
-	open(FILE,">>$filename");
+	open(my $fh, ">>$filename");
 	unless($gb->{"header_signature"}){
-	    print FILE "start,end,G+C,";
-	    foreach(@allkey){ print FILE "$_,"; }
+	    print $fh "start,end,G+C,";
+	    foreach(@allkey){ print $fh "$_,"; }
 	    $gb->{"header_signature"} = 1;
-	    print FILE "\n";
+	    print $fh "\n";
         }
-	print FILE "$start,$end,$GC,";
-	foreach(@allkey){ print FILE (exists $oe{$_}) ? "$oe{$_}," : "0,"; }
-	print FILE "\n";
-	close(FILE);
+	print $fh "$start,$end,$GC,";
+	foreach(@allkey){ print $fh (exists $oe{$_}) ? "$oe{$_}," : "0,"; }
+	print $fh "\n";
+	close($fh);
     }
 }
 
@@ -238,10 +406,11 @@ sub signature {
    -window   size (bp) of sliding window by which sequences are sampled (default: 5000)
    -step     step size (bp) of sliding window (default: 5000)
    -method   the distance measure to be used. (default: 'mahalanobis')
-             This must be one of "euclidean", "delta" or "mahalanobis". 
+             This must be one of "mahalanobis", "delta", "euclidean", "maximum", 
+             "manhattan", "canberra", "binary" or "minkowski".
 
   Examples:
-   &signature_dist($gb_plasmid, $gb_chromosome, -wordlength=>2, -window=>50000, -method=>'delta');
+   &signature_dist($gb_plasmid, $gb_chromosome, -wordlength=>2, -window=>50000, -step=>50000, -method=>'delta');
    calculates the average absolute dinucleotide relative abundance difference (delta-distance) 
    between a plasmid and non-overlapping 50-kb chromosomal segments (Campbell A et al., 1999).
 
@@ -261,27 +430,31 @@ sub signature {
 
 sub signature_dist {
     &opt_default(output=>'stdout', wordlength=>2, window=>5000, step=>5000, method=>'mahalanobis');
-    my @args = opt_get(@_);
-    my $query = opt_as_gb(shift @args);
-    my $sbjct = opt_as_gb(shift @args);
-    my $output = opt_val("output");
+    my @args       = opt_get(@_);
+    my $query      = opt_as_gb(shift @args);
+    my $sbjct      = opt_as_gb(shift @args);
+    my $output     = opt_val("output");
     my $wordlength = opt_val("wordlength");
-    my $window = opt_val("window");
-    my $step = opt_val("step");
-    my $method = opt_val("method");
+    my $window     = opt_val("window");
+    my $step       = opt_val("step");
+    my $method     = opt_val("method");
+    my $rand       = time() . rand(1000);
+ 
+#    system('echo -n > query.csv > sbjct.csv');
 
-    system('echo -n > query.csv > sbjct.csv');
-    &signature($query, -output=>"f", -filename=>'query.csv', -wordlength=>$wordlength);
-    for(my $i = 1; $i <= length($sbjct->{SEQ}) - $window; $i += $step){ &signature($sbjct, -output=>"f", -filename=>'sbjct.csv', -wordlength=>$wordlength, -start=>$i, -end=>$i+$window); }
+    &signature($query, -output=>"f", -filename=>"/tmp/query$rand.csv", -wordlength=>$wordlength);
+    for(my $i = 1; $i <= length($sbjct->{SEQ}) - $window; $i += $step){ 
+	&signature($sbjct, -output=>"f", -filename=>"/tmp/sbjct$rand.csv", -wordlength=>$wordlength, -start=>$i, -end=>$i+$window); 
+    }
     
     my $rcmd = new Rcmd;
     my @result = $rcmd->exec(
-			     qq/
+			     qq|
 			     method = "$method"
-			     f = "query.csv"
+			     f = "/tmp/query$rand.csv"
 			     X = read.csv(f)
 			     X = X[,-c(1,2,3,length(X))]
-			     f = "sbjct.csv"
+			     f = "/tmp/sbjct$rand.csv"
 			     Y = read.csv(f)
 			     Y = Y[,-c(1,2,3,ncol(Y))]
 			     if(method == "mahalanobis"){ 
@@ -301,8 +474,11 @@ sub signature_dist {
 			     D = dX
 			     P = ifelse(is.numeric(dY), mean(dY > dX), NA)
 			     as.vector(c(D, P))
-			     /
+			     |
 			     );
+
+    unlink("/tmp/query$rand.csv");
+    unlink("/tmp/sbjct$rand.csv");
 #    print "@result \n";
     my $distance = sprintf "%.4f", shift @result;
     my $p_value = sprintf "%.4f", shift @result;
