@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Primitive.pm,v 1.1 2002/07/30 17:44:27 gaou Exp $
@@ -90,13 +90,15 @@ my %CodonTable = (
 
 =head2 translate
 
-  Name: translate  -   translate nucleotide sequence to amino acid sequence
+  Name: translate   -   translate nucleotide sequence to amino acid sequence
 
   Description:
     This method translates a given nucleotide sequence into amino acid 
     sequence, using standard codon table. Change to the codon table or
     custom codon table can be supplied as optional arguments. Stop codons
     are denoted by '/' by default.
+
+    eg. translate('ctggtg');  # returns 'LV'
     
   Usage : 
     $amino = translate($nucleotide);
@@ -107,9 +109,12 @@ my %CodonTable = (
     # to use custom codon table
     $amino = translate($nucleotide, %customCodonTable);
 
- Options:
+  Options:
     Custom codon table can be optionally used (see above).
 
+  REST: 
+      http://rest.g-language.org/translate/ctggtg
+
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
@@ -149,6 +154,27 @@ sub translate {
 
 __DATA__
 
+
+=head2 complement
+
+   Name: complement   -   get the complementary nucleotide sequence
+
+   Description:
+         Given a sequence, returns its complement.
+
+         eg. complement('atgc');  # returns 'gcat'
+
+   REST: 
+      http://rest.g-language.org/complement/atgc 
+
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+  History:
+    20010727-01 initial posting
+
+=cut
+
 sub complement {
     my $nuc = reverse(shift);
 
@@ -484,6 +510,9 @@ sub shuffleseq{
     -speed      controls the time to display one base-pair.
                 default is 0.05 (second)
 
+  REST:
+    http://rest.g-language.org/NR_029721/DoubleHelix/speed=0
+
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
@@ -494,9 +523,9 @@ sub shuffleseq{
 
 sub DoubleHelix{
     SubOpt::opt_default(speed=>0.05);
-    my @args = SubOpt::opt_get(@_);  
-    my $gb = SubOpt::opt_as_gb(shift @args);
-    my $speed = SubOpt::opt_val("speed");
+    my @args = opt_get(@_);  
+    my $gb = opt_as_gb(shift @args);
+    my $speed = opt_val("speed");
 
     $| = 1;
 
@@ -505,12 +534,13 @@ sub DoubleHelix{
     my (@dist)   = qw/0 2 3 4 4 4 3 2 0 0 2 3 4 4 4 3 2 0/;
 
     foreach my $base (split(//, $gb->{SEQ})){
-        print "         ", q/ /x$offset[($i%scalar(@offset))];
-	print uc($base);
+        my $msg = "         " .  q/ /x$offset[($i%scalar(@offset))];
+	$msg .= uc($base);
 	$i ++;
-	print q/-/x$dist[($i%scalar(@dist))];
-	print uc(complement($base)), "\n";
-	select(undef,undef,undef,$speed);
+	$msg .= q/-/x$dist[($i%scalar(@dist))];
+	$msg .=  uc(complement($base)) .  "\n";
+	msg_send($msg);
+	select(undef,undef,undef,$speed) unless($speed == 0);
     }
 }
 
