@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Tandem.pm,v 1.1.1.1 2002/04/02 20:25:42 gaou Exp $
@@ -38,7 +38,6 @@ use SelfLoader;
 our @EXPORT = qw(
 		 find_tandem
 		 foreach_tandem
-		 graphical_LTR_search
 );
 
 
@@ -322,84 +321,4 @@ sub _print_tandem{
 }
 
 
-# graphical_LTR_search ver.20010906-01
-# Author: Kazuharu Arakawa
-# Usage: NULL = &graphical_LTR_search(pointer string seq); 
-# Options:
-# -length    length of LTR to search (default:64)
-# -filename  output filename (default:'LTR.png')
-# -output    "g" for graph, "show" for display 
-# Description:
-#  Converts a sequence to a png image in order to find LTR.
-#  This method is useful because if LTR is present, you can
-#  see pixels aligned in several rows at the locus.
-#  A is shown in red, T is shown in green, 
-#  G is shown in yellow, and C is shown in blue. 
-# Requirements:
-#  SubOpt.pm
-#  GD.pm
-
-sub graphical_LTR_search {
-    &opt_default(length=>64, filename=>"LTR.png", output=>"show");
-    my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
-    my $seq = \$gb->{SEQ};
-    my $width = opt_val("length");
-    my $height = int((length($$seq)+1)/$width)+1;
-    die("\nERROR at graphical_LTR_search: sequence too long!\n\n\n")
-	if ($height > 15000);
-    my $output = opt_val("output");
-    my $filename = opt_val("filename");
-    my $topmargin = 50;
-    my $sidemargin = 50;
-    my $im = new GD::Image($width + $sidemargin * 2, 
-			   $height + $topmargin + 20);
-
-    my $white = $im->colorAllocate(255,255,255);
-    my $black = $im->colorAllocate(0,0,0);
-    my $red = $im->colorAllocate(255,0,0);
-    my $yellow = $im->colorAllocate(255,255,0);
-    my $green = $im->colorAllocate(0,150,0);
-    my $blue = $im->colorAllocate(0,0,255);
-
-    my ($x, $y);
-    my $count = 0;
-    for ($y = $topmargin; $y <= $height + $topmargin; $y ++){
-	for ($x = 0; $x <= $width; $x ++){
-	    my $color=$white;
-	    $color=$red if (substr($$seq, $count, 1) eq 'a');
-	    $color=$yellow if (substr($$seq, $count, 1) eq 'g');
-	    $color=$green if (substr($$seq, $count, 1) eq 't');
-	    $color=$blue if (substr($$seq, $count, 1) eq 'c');
-	    
-	    $im->setPixel($x + $sidemargin,$y,$color);
-	    last if ($count == length($$seq));
-	    $count ++;
-	}
-    }
-    for ($y = $topmargin; $y <= $height + $topmargin; $y += $sidemargin){
-	my $num = ($y - $topmargin) * $width;
-	$im->string(gdTinyFont, 6, $y, "$num", $black);
-    }
-    $im->string(gdSmallFont, 5,  5, "LTR search:", $black);
-    $im->string(gdSmallFont, 20, 20, "width = $width", $black);
-    $im->string(gdSmallFont, $sidemargin + 25 + $width, $topmargin + 10, 
-		"A", $red);
-    $im->string(gdSmallFont, $sidemargin + 25 + $width, $topmargin + 30,
-		"T", $yellow);
-    $im->string(gdSmallFont, $sidemargin + 25 + $width, $topmargin + 50,
-		"G", $green);
-    $im->string(gdSmallFont, $sidemargin + 25 + $width, $topmargin + 70, 
-		"C", $blue);
-    
-    mkdir ("graph", 0777);
-    open(OUT, '>graph/' . $filename);
-    binmode OUT;
-    print OUT $im->png;
-    close(OUT);
-
-    msg_gimv("gimv graph/$filename") if ($output eq 'show');
-}
-
-
 1;
