@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 # G-language GAE is free software; you can redistribute it and/or
@@ -42,7 +42,6 @@ our @EXPORT = qw(
 #       Perldoc
 #:::::::::::::::::::::::::::::::::
 
-
 =head1 NAME
 
   G::Shell::EUtils - G-language Shell helper module for searching NCBI.
@@ -58,7 +57,7 @@ our @EXPORT = qw(
 #          Variables
 #::::::::::::::::::::::::::::::
 
-my $esearch = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?';
+my $esearch  = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?';
 my $esummary = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?';
 my %einfo;
 
@@ -74,7 +73,6 @@ map {$einfo{$_} = 1} qw(
 #    Let the code begin...
 #::::::::::::::::::::::::::::::
 
-
 =head2 pubmed
 
   Name: pubmed   -   search PubMed in G-language Shell
@@ -104,7 +102,7 @@ sub pubmed {
     my $keyword = shift;
 
     unless($keyword){
-	help("pubmed");
+	help('pubmed');
 	return;
     }
 
@@ -172,7 +170,7 @@ sub entrez {
     my $keyword = shift;
 
     unless(length($db) || $keyword){
-	help("entrez");
+	help('entrez');
 	return;
     }
 
@@ -181,16 +179,15 @@ sub entrez {
 	return;
     }
 
-
     my $ua = new LWP::UserAgent;
 
     my %params = (
-		  email => 'gaou@sfc.keio.ac.jp',
-		  tool => 'g-language.org',
-		  db => $db,
-		  retmax => 10,
-		  term => $keyword,
+		  email   => 'gaou@sfc.keio.ac.jp',
+		  tool    => 'g-language.org',
+		  db      => $db,
+		  retmax  => 10,
 		  retmode => 'xml',
+		  term    => $keyword,
 		  );
 
     my $url = $esearch . join('&', map {"$_=$params{$_}"} keys %params);
@@ -201,25 +198,25 @@ sub entrez {
     if ($response->is_success) {
 	my $xml = XMLin($response->content);
 
-	my $entryText = "entries";
+	my $entryText = 'entries';
 	$entryText = "entry" if ($xml->{Count} == 1);
 
 	printf "    %d %s found in %s: (Showing up to 10 hits)\n\n", $xml->{Count}, $entryText, uc($db);
 
 	my $id;
 	if(ref($xml->{IdList}->{Id}) eq 'ARRAY'){
-	    $id = join(",", @{$xml->{IdList}->{Id}});
+	    $id = join(',', @{$xml->{IdList}->{Id}});
 	}else{
 	    $id = $xml->{IdList}->{Id};
 	}
 
 	my %params = (
-		      email => 'gaou@sfc.keio.ac.jp',
-		      tool => 'g-language.org',
-		      db => $db,
-		      retmax => 10,
+		      email   => 'gaou@sfc.keio.ac.jp',
+		      tool    => 'g-language.org',
+		      db      => $db,
+		      retmax  => 10,
 		      retmode => 'xml',
-		      id => $id,
+		      id      => $id,
 		      );
 
 	my $url = $esummary . join('&', map {"$_=$params{$_}"} keys %params);
@@ -282,7 +279,6 @@ sub entrez {
 		    $i ++;
 		}
 	    }
-
 	} else {
 	    print $response->error_as_HTML;
 	}
