@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: EPCR.pm,v 1.1.1.1 2002/04/02 20:25:44 gaou Exp $
@@ -37,6 +37,7 @@ use LWP::Simple;
 
 our @EXPORT = qw(
 		 ws
+		 keggapi
 		 togoWS
 		 );
 
@@ -67,6 +68,40 @@ __DATA__
 #:::::::::::::::::::::::::::::::::
 
 
+=head2 keggapi
+
+ Name: keggapi   -   runs KEGG API
+
+ Description:
+    This is a wrapper around KEGG API REST wrapper, provided at
+    http://rest.g-language.org/keggapi/.
+
+    Refer to the official KEGG API web site for detailed documentations.
+    http://www.genome.jp/kegg/soap/doc/keggapi_manual.html
+    
+ Usage:
+    ex. 
+    @list_of_services = keggapi();
+
+    @list_of_pathways = keggapi('get_pathways_by_compounds', 'C00668');
+    #Note that cpd: for compound ID is optional. Likewise for ec:, gl:, dr:
+
+ Options:
+    None.
+
+ Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+
+ History:
+   20100123-01 initial posting
+
+=cut
+
+sub keggapi{
+    my $program = shift;
+    return readFile('http://rest.g-language.org/keggapi/' . $program . '/' . join('/', @_), 1);
+}
+
 =head2 togoWS
 
  Name: togoWS   -   runs togoWS REST web services
