@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: Rcmd.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -34,23 +34,19 @@ use base qw(Rcmd::Clustering Rcmd::Normality);
 use autouse 'Carp'=>qw(croak);
 use autouse 'File::Copy'=>qw(copy);
 
-our $VERSION = '1.21';
-
-srand(time ^ $$ ^ unpack "%L*", `ps axww | gzip`);
+use File::Temp;
+our $VERSION = '1.22';
 
 ######################################
 #       Let the code begin...
 ######################################
 
-my $save = '--no-save';
-my $output = '';
-my @tmpdat = ();
-
 sub new {
-    my $this = shift;
-    $save = shift; 
-    $output = shift;
-    my $uniq = rand(9999999999);
+    my $pkg = shift;
+    my $save = shift || '--no-save'; 
+    my $output = shift;
+
+    my $uniq = time() . rand(1000);
     my $cmd = "/tmp/$uniq.cmd";
     my $log = "/tmp/$uniq.log";
 
@@ -67,14 +63,19 @@ sub new {
 	return;
     }
 
-    bless { 
-	cmd => $cmd, 
-	log => $log,
-	tmpcmd => $tmpcmd,
-	tmplog => $tmplog,
-	sessioncmd => $cmd,
-	sessionlog => $log
-    }
+    my $this = {};
+
+    $this->{cmd}        = $cmd; 
+    $this->{log}        = $log;
+    $this->{tmpcmd}     = $tmpcmd;
+    $this->{tmplog}     = $tmplog;
+    $this->{sessioncmd} = $cmd;
+    $this->{sessionlog} = $log;
+    $this->{output}     = $output;
+    $this->{save}       = $save;
+    $this->{tmpdat}     =[];
+
+    return bless $this;
 }
 
 sub set_mode{
@@ -95,17 +96,14 @@ sub set_mode{
 
 sub DESTROY{
     my $this = shift;
-    if (length $output){
-	copy($this->{cmd}, $output);
+    if (length $this->{output}){
+	copy($this->{cmd}, $this->{output});
     }
     unlink $this->{cmd};
     unlink $this->{log};
     unlink $this->{tmpcmd};
     unlink $this->{tmplog};
 
-    foreach my $file (@tmpdat){
-	unlink $file;
-    }
 }
 
 sub array{
@@ -113,14 +111,13 @@ sub array{
     my $val = shift;
     my @array = @_;
 
-    my $uniq = rand(9999999999);
-    open(OUT, ">/tmp/$uniq.dat") || die($!);
-    print OUT join(",", @array);
-    close(OUT);
-    open(CMD, '>>' . $this->{cmd});
-    print CMD "$val <- scan\(\"/tmp/$uniq.dat\", sep\=\",\"\)\n";
-    close(CMD);
-    push(@tmpdat, "/tmp/$uniq.dat");
+    my $fh = File::Temp->new(SUFFIX=>'.dat');
+    print $fh join(",", @array);
+
+    open(my $fh2, '>>' . $this->{cmd});
+    print $fh2 "$val <- scan\(\"" . $fh->filename . "\", sep\=\",\"\)\n";
+    close($fh2);
+    push(@{$this->{tmpdat}}, $fh);
 }
 
 sub sarray{
@@ -128,15 +125,13 @@ sub sarray{
     my $val = shift;
     my @array = @_;
 
-    my $uniq = rand(9999999999);
-    open(OUT, ">/tmp/$uniq.dat") || die($!);
-    print OUT join(" ", @array), "\n";
-    close(OUT);
+    my $fh = File::Temp->new(SUFFIX=>'.dat');
+    print $fh join(" ", @array), "\n";
 
-    open(CMD, '>>' . $this->{cmd});
-    print CMD "$val <- scan\(\"/tmp/$uniq.dat\"\, character\(\)\)\n";
-    close(CMD);
-    push(@tmpdat, "/tmp/$uniq.dat");
+    open(my $fh2, '>>' . $this->{cmd});
+    print $fh2 "$val <- scan\(\"" . $fh->filename . "\"\, character\(\)\)\n";
+    close($fh2);
+    push(@{$this->{tmpdat}}, $fh);
 }
     
 
@@ -154,15 +149,15 @@ sub exec{
     }
     my $data = '';
 
-    open(CMD, '>>' . $this->{cmd});
-    print CMD $request;
-    close(CMD);
+    open(my $fh, '>>' . $this->{cmd});
+    print $fh $request;
+    close($fh);
 
-    system("/usr/bin/env R $save --slave < "
-	   . $this->{cmd} . " 2>&1 " . $this->{log});
+    system("/usr/bin/env R " . $this->{save} . " --slave < "
+	   . $this->{cmd} . " >& " . $this->{log});
 
-    open(DATA, $this->{log});
-    while(<DATA>){
+    open(my $fh2, $this->{log});
+    while(<$fh2>){
 	if (/\[(\d+).*\] +(.*)/){
 	    if ($1 > 1){
 		$data .= ' ' . $2;
@@ -171,20 +166,20 @@ sub exec{
 	    }
 	}elsif(/Error/){
 	    print STDERR $_;
-	    while(<DATA>){
+	    while(<$fh2>){
 		print STDERR $_;
 	    }
 	    die("Error in R, exiting...\n");
 	}elsif(/Warning/){
 	    print STDERR $_;
-	    while(<DATA>){
+	    while(<$fh2>){
 		print STDERR $_;
 		last;
 	    }
 	    warn("Warning in R...\n");
 	}
     }
-    close(DATA);
+    close($fh2);
 
     if (wantarray()){
 	return split(/\s+/,$data);
