@@ -1,7 +1,7 @@
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # This file is part of G-language Genome Analysis Environment package
 #
-#     Copyright (C) 2001-2009 Keio University
+#     Copyright (C) 2001-2010 Keio University
 #:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 # 
 #   $Id: SubOpt.pm,v 1.3 2002/07/30 17:40:56 gaou Exp $
@@ -78,10 +78,9 @@ sub opt_as_gb {
     my $unknown = shift @_;
     my $ref = \$unknown;
     my $ref2 = ref $unknown;
-    my $new_gb = new G::IO('blessed');
 
     if ($ref2 =~ /Bio::Seq/){
-	return new G::IO($unknown, "bioperl", "no msg");
+	return new G::IO($unknown, 'bioperl', 'no msg');
     }elsif ("$unknown" =~ /\=/){
 	if (defined $unknown->{SEQ}){
 	    return $unknown;
@@ -90,14 +89,23 @@ sub opt_as_gb {
 	    return '';
 	}
     } elsif (ref $unknown eq 'SCALAR') {
+	my $new_gb = new G::IO('blessed');
 	$new_gb->{SEQ} = lc($$unknown);
 	return $new_gb;
     } elsif (ref $unknown eq 'REF') {
+	my $new_gb = new G::IO('blessed');
 	$new_gb->{SEQ} = lc($$unknown);
 	return $new_gb;
     } elsif (ref $ref eq 'SCALAR'){
-	$new_gb->{SEQ} = lc($unknown);
-	return $new_gb;
+	if(-e $unknown){
+	    return new G::IO($unknown, 'no msg');
+	}elsif(length($unknown) < 25){
+	    return new G::IO($unknown, 'no msg');
+	}else{
+	    my $new_gb = new G::IO('blessed');
+	    $new_gb->{SEQ} = lc($unknown);
+	    return $new_gb;
+	}
     }else{
 	croak("Error 4 in SubOpt::opt_as_gb():\n  Given value is not a valid sequence.\n");
 	return '';
